@@ -128,6 +128,8 @@ class ModuleNotAvailable(object):
 
 
 class XContext(ABC):
+    minimum_alignment = 1
+
     def __init__(self):
         self._kernels = MinimalDotDict()
         self._buffers = []
@@ -186,7 +188,7 @@ class XContext(ABC):
 
 
 class XBuffer(ABC):
-    def __init__(self, capacity=1048576, context=None):
+    def __init__(self, capacity=1048576, context=None, default_alignment=None):
 
         if context is None:
             self.context = self._make_context()
@@ -194,34 +196,46 @@ class XBuffer(ABC):
             self.context = context
         self.buffer = self._new_buffer(capacity)
         self.capacity = capacity
+        if default_alignment is None:
+            default_alignment = self.context.minimum_alignment
+        self.default_alignment = default_alignment
         self.chunks = [Chunk(0, capacity)]
 
     @abstractmethod
     def _make_context(self):
         "return a default context"
 
-    def allocate(self, size, alignment=1):
+    def allocate(self, size, align=False):
         # find available free slot
         # and update free slot if exists
-        sizepa = size + alignment - 1
+        if align:
+            alignment = self.default_alignment
+        else:
+            alignment = 1
+        # sizepa = size + alignment - 1
         for chunk in self.chunks:
-            if sizepa <= chunk.size:
-                offset = chunk.start
-                chunk.start += sizepa
+            offset = _align(chunk.start, alignment)
+            newend = offset + size
+            if chunk.end >= newend:
+                chunk.start = newend
                 if chunk.size == 0:
                     self.chunks.remove(chunk)
-                return _align(offset, alignment)
+                return offset
 
         # no free slot check if can be allocated then try to grow
+        sizepa = size + alignment - 1
         if sizepa > self.capacity:
             self.grow(sizepa)
         else:
             self.grow(self.capacity)
 
         # try again
-        return self.allocate(sizepa)
+        return self.allocate(size, align=align)
 
     def grow(self, capacity):
+        """
+        Add capacity to buffer
+        """
         oldcapacity = self.capacity
         newcapacity = self.capacity + capacity
         newbuff = self._new_buffer(newcapacity)
@@ -393,3 +407,80 @@ class Method:
             return kernel(*args, **kwargs)
 
         return a_method
+
+
+def get_context_from_string(ctxstr):
+    import xobjects as xo
+
+    if ctxstr is None:
+        return xo.ContexCPU()
+    else:
+        ll = ctxstr.split(":")
+        if len(ll):
+            ctxtype = ll[0]
+            option = []
+        else:
+            ctxtype, options = ctxstr.split(":")
+            option = options.split(",")
+    if ctxtype == "ContextCpu":
+        if len(option) == 0:
+            return xo.ContextCpu()
+        else:
+            return xo.ContextCpu(omp_num_threads=int(option[0]))
+    elif ctxtype == "ContextCupy":
+        if len(option) == 0:
+            return xo.ContextCupy()
+        else:
+            return xo.ContextCupy(device=int(option[0]))
+    elif ctxtype == "ContextPyopencl":
+        if len(option) == 0:
+            return xo.ContextPyopencl()
+        else:
+            return xo.ContextPyopencl(device=option[0])
+    else:
+        raise ValueError(f"Cannot create context from `{ctxstr}`")
+
+
+def get_test_contexts():
+    import os
+    import xobjects as xo
+
+    ctxstr = os.environ.get("XOBJECT_TEST_CONTEXTS")
+    if ctxstr is None:
+        yield xo.ContextCpu()
+        yield xo.ContextCpu(omp_num_threads=2)
+        if xo.ContextCupy in xo.context.available:
+            yield xo.ContextCupy()
+        if xo.ContextPyopencl in xo.context.available:
+            yield xo.ContextPyopencl()
+    elif ctxstr == "all":
+        yield xo.ContextCpu()
+        yield xo.ContextCpu(omp_num_threads=2)
+        if xo.ContextCupy in xo.context.available:
+            yield xo.ContextCupy()
+        if xo.ContextPyopencl in xo.context.available:
+            for dd in xo.ContextPyopencl.get_devices():
+                yield xo.ContextPyopencl(device=dd)
+
+    else:
+        for cc in ctxstr.split(";"):
+            yield get_context_from_string(cc)
+
+
+def get_user_context():
+    """
+    Get the context specfied by the enviroment variable XOBJECT_USER_CONTEXT.
+    If not present use ContextCpu().
+
+    Examples:
+       ContextPyopencl:0.0  -> ContextPyopencl(device="0.0")
+       ContextPyopencl:1.0  -> ContextPyopencl(device="1.0")
+       ContextPyopencl      -> ContextPyopencl()
+       ContextCpu           -> ContextCpu()
+       ContextCpu:2         -> ContextCpu(omp_num_threads=2)
+       ContextCupy:0        -> ContextCupy(device=0)
+    """
+    import os
+
+    ctxstr = os.environ.get("XOBJECT_USER_CONTEXT")
+    return get_context_from_string(ctxstr)
